import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import java.util.function.Function;

public class CsvToHtml {
    private Function<String[], CsvEntry> cellsToEntry;

    public CsvToHtml(Function<String[], CsvEntry> cellsToEntry) {
        this.cellsToEntry = cellsToEntry;
    }

    private static String[] splitLine(String line) {
    	return line.split(",");
    }

    public void convert(Path in, Path out) {
    	
    	// check, if file is regular
        if (!Files.isRegularFile(in)) {
            System.out.println("input file not existent/readable!");
            return;
        }
        
        // check, if output file already exists
        if (Files.exists(out)) {
            System.out.println("output file already exists!");
            return;
        }

        // create output file
        try {
            Files.createFile(out);
        } catch (IOException e) {
            System.out.println("output file could not be created!");
        }

        try {
        	// add HTML Head
            Files.writeString(out, "<html><body><table>\n",
                    StandardOpenOption.APPEND);
            
            // split input into cells
            Files.lines(in).map(CsvToHtml::splitLine).filter(e -> e != null)
            
            // convert cells to PhoneBookEntries (filter for errors == null)
                    .map(cellsToEntry).filter(e -> e != null)
                    
                    // add lines sequentially
                    .map(CsvEntry::toHtml).forEach(line -> {
                        try {
                            Files.writeString(out, line + "\n",
                                    StandardOpenOption.APPEND);
                        } catch (IOException e) {
                            System.out.println("could not read/write!");
                        }
                    });
            
            // add HTML Head (closing)
            Files.writeString(out, "</table></body></html>\n",
                    StandardOpenOption.APPEND);
            // StandardOpenOption to define behaviour (create file, just write, etc.)
        } catch (IOException e) {
            System.out.println("could not read/write!");
        }
    }

    public static void main(String[] args) {
    	if (args.length != 2) {
            System.out.println("usage: csvtohtml in out");
            return;
        }

        CsvToHtml cth = new CsvToHtml(PhoneBookEntry::cellsToEntry);
        cth.convert(Paths.get(args[0]), Paths.get(args[1]));
    }
}
